package io.skore.smooch_plugin

import android.app.Activity
import android.content.Intent
import android.util.Log
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.skore.smooch.util.LangPreference
import io.smooch.core.*
import java.util.*

class SmoochPlugin(private val activity: Activity, private val channel: MethodChannel) : MethodCallHandler {
    private val TAG = SmoochPlugin::class.java.simpleName

    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "smooch_plugin")
            channel.setMethodCallHandler(SmoochPlugin(registrar.activity(), channel))
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "init" -> {
                val appId = call.argument<String>("appId")
                val userId = call.argument<String>("userId")
                val smoochToken = call.argument<String>("smoochToken")
                if (appId.isNullOrBlank() || userId.isNullOrBlank() || smoochToken.isNullOrBlank()) {
                    result.error("ERROR", "Invalid arguments", null)
                    return
                }
                init(appId, userId, smoochToken, result)
            }
            "show" -> {
                val lang = call.argument<String>("lang")
                val userProp = call.argument<HashMap<String, Any>>("properties")
                show(lang, userProp, result)
            }
            "logout" -> {
                Smooch.logout {
                    Log.d(TAG, "Smooch logout status: ${it.status} error: ${it.error}")
                }
            }
            else -> result.notImplemented()
        }
    }

    private fun init(appId: String, userId: String, smoochToken: String, result: Result) {
        val initStatus = Smooch.getInitializationStatus()
        if (initStatus != InitializationStatus.Success) {
            Smooch.init(activity.application, Settings(appId)) { initResponse ->
                Log.d(TAG, "Smooch init status: ${initResponse.status}")
                if (initResponse.status == 200) {

                    val lastLoginResult = Smooch.getLastLoginResult()
                    if (lastLoginResult != LoginResult.Success) {
                        Smooch.login(userId, smoochToken) { loginResponse ->
                            Log.d(TAG, "Smooch login status: ${loginResponse.status}")
                            if (loginResponse.status == 200) {
                                result.success(0)
                            } else {
                                result.success(-1)
                            }
                        }
                    }
                } else {
                    result.success(-1)
                }
            }
        }

        addDelegate()
    }

    private fun addDelegate() {
        Smooch.getConversation()?.apply {
            delegate = object : Conversation.Delegate {
                override fun onInitializationStatusChanged(initStatus: InitializationStatus?) {}

                override fun onSmoochHidden() {}

                override fun onPaymentProcessed(msgAction: MessageAction?, paymentStatus: PaymentStatus?) {}

                override fun onMessageSent(msg: Message?, msgUploadStatus: MessageUploadStatus?) {}

                override fun onLoginComplete(loginResult: LoginResult?) {}

                override fun onSmoochShown() {}

                override fun onCardSummaryLoaded(summary: CardSummary?) {}

                override fun onSmoochConnectionStatusChanged(connectionStatus: SmoochConnectionStatus?) {}

                override fun onMessagesReceived(conversation: Conversation?, msgList: MutableList<Message>?) {}

                override fun onConversationEventReceived(conversationEvent: ConversationEvent?) {}

                override fun onMessagesReset(conversation: Conversation?, msgList: MutableList<Message>?) {}

                override fun shouldTriggerAction(messageAction: MessageAction?): Boolean {
                    messageAction?.apply {
                        return when (type.toLowerCase(Locale.ENGLISH)) {
                            "link",
                            "webview" -> {
                                channel.invokeMethod("handleMessageAction", uri)
                                false
                            }
                            else -> {
                                uri.contains("viaapp_terms", true)
                            }
                        }
                    }
                    return true
                }

                override fun onLogoutComplete(logoutResult: LogoutResult?) {
                    channel.invokeMethod("logoutComplete", true)
                }

                override fun onUnreadCountChanged(conversation: Conversation?, unreadCount: Int) {
                    channel.invokeMethod("updateChatUnreadCount", unreadCount)
                }
            }

            messageModifierDelegate = object : Conversation.MessageModifierDelegate {
                override fun beforeSend(message: Message?): Message? {
                    return message
                }

                override fun beforeNotification(message: Message?): Message? {
                    return message
                }

                override fun beforeDisplay(message: Message?): Message? {
                    message?.text?.let { txt ->
                        if (txt.equals("iniciar", true) ||
                                txt.equals("session", true) ||
                                txt.contains("token:") ||
                                txt.contains("token2:")) {
                            return null
                        }
                    }
                    return message
                }
            }
        }
    }

    private fun show(lang: String?, userProp: HashMap<String, Any>?, result: Result) {
        LangPreference.setUserLang(activity, lang)

        channel.invokeMethod("shouldSendStart", null, object : Result {
            override fun notImplemented() {
                Log.d(TAG, "notImplemented")
                result.success(-1)
            }

            override fun error(p0: String?, p1: String?, p2: Any?) {
                Log.d(TAG, "error $p0 $p1")
                result.success(-1)
            }

            override fun success(isSuccess: Any?) {
                Log.d(TAG, "success")

                val msg = Message("iniciar")
                Smooch.getConversation()?.sendMessage(msg)

                result.success(0)
            }
        })

        if (userProp is Map<*, *>) {
            @Suppress("UNCHECKED_CAST")
            User.getCurrentUser().addProperties(userProp as Map<String, Any>)
        }

        activity.startActivity(Intent(activity, SmoochActivity::class.java))
    }
}
